/*
keyconfig.js - Script for creating a user keyconfiguration.
Copyright  2012 Svengali

Special thanks to Eric Walch, Micha, PhantorGorth, El Viejo and Gimi

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA 02110-1301, USA.

More infos about key handling (and it's inconsistancies) in JS
 can be found at: http://unixpapa.com/js/key.html

*/
(function() {
  this.KC_Version = VKI_version;
  /* Keyconfig settings.
  * Adding new keys requires a changed table in keyconfig.html.
  *
  * [0]=Default, [1]=Alternative, [2]=BBC
  * The value is array of [keycode, clashtype, coloring]
  * clashtype is used to mark duplicated entries (bitmask):
  * 1=Inflight, 2=Docked, 4=Docking Comp, 8=GUI (other than inflight views), 16=Pause
  * coloring just selects the background-color based on this.KC_StyleAtt:
  * 1=Weapon, 2=Combat, 3=Nav, 4=Views, 5=Equipment, 6=Options
  */
  this.KC_Default = {
    key_roll_left: [[253,1,3],[253,1,3],[",",1,3]],
    key_roll_right: [[252,1,3],[252,1,3],[".",1,3]],
    key_pitch_forward: [[255,1,3],[255,1,3],["s",1,3]],
    key_pitch_back: [[254,1,3],[254,1,3],["x",1,3]],
    key_yaw_left: [[",",1,3],[",",1,3],[253,1,3]],
    key_yaw_right: [[".",1,3],[".",1,3],[252,1,3]],
    key_increase_speed: [["w",1,3],["w",1,3],[32,1,3]],
    key_decrease_speed: [["s",1,3],["s",1,3],["/",1,3]],
    key_inject_fuel: [["i",1,3],["i",1,3],["i",1,3]],
    key_fire_lasers: [["a",1,1],["a",1,1],["a",1,1]],
    key_weapons_online_toggle: [["_",1,1],["W",1,1],["_",1,1]],
    key_launch_missile: [["m",1,1],["m",1,1],["m",1,1]],
    key_next_missile: [["y",1,1],["y",1,1],["y",1,1]],
    key_ecm: [["e",1,1],["e",1,1],["e",1,1]],
    key_prime_equipment: [["N",1,5],["[",1,5],["N",1,5]],
    key_activate_equipment: [["n",1,5],["]",1,5],["n",1,5]],
    key_mode_equipment: [["b",1,5],["\\",1,5],["b",1,5]],
    key_target_incoming_missile: [["T",1,1],["T",1,1],["T",1,1]],
    key_target_missile: [["t",1,1],["t",1,1],["t",1,1]],
    key_untarget_missile: [["u",1,1],["u",1,1],["u",1,1]],
    key_ident_system: [["r",1,2],["r",1,2],["r",1,2]],
    key_scanner_zoom: [["z",1,2],[">",1,2],["z",1,2]],
    key_scanner_unzoom: [["Z",1,2],["<",1,2],["Z",1,2]],
    key_launch_escapepod: [[27,1,5],[27,1,5],[27,1,5]],
    key_energy_bomb: [["\t",1,1],["|",1,1],["\t",1,1]],
    key_galactic_hyperspace: [["g",1,3],["G",1,3],["g",1,3]],
    key_hyperspace: [["h",1,3],["H",1,3],["h",1,3]],
    key_jumpdrive: [["j",1,3],["j",1,3],["j",1,3]],
    key_dump_cargo: [["d",1,5],[306,1,5],["d",1,5]],
    key_rotate_cargo: [["R",1,5],["R",1,5],["R",1,5]],
    key_autopilot: [["c",1,3],["D",1,3],["c",1,3]],
    key_autodock: [["C",1,3],["d",1,3],["C",1,3]],
    key_docking_clearance_request: [["L",1,3],[":",1,3],["L",1,3]],
    key_snapshot: [["*",15,6],[249,15,6],["*",15,6]],
    key_docking_music: [["s",4,6],[307,4,6],["s",4,6]],
    key_advanced_nav_array: [["^",10,4],["N",10,4],["^",10,4]],
    key_map_home: [[303,10,4],[303,10,4],[302,10,4]],
    key_map_info: [["i",10,4],["M",10,4],["i",10,4]],
    key_pausebutton: [["p",31,6],["p",31,6],["p",31,6]],
    key_show_fps: [["F",31,6],["F",31,6],["F",31,6]],
    key_mouse_control: [["M",31,6],[308,31,6],["M",31,6]],
    key_hud_toggle: [["o",16,6],[304,16,6],["o",16,6]],
    key_comms_log: [["`",1,2],["\t",1,2],["`",1,2]],
    key_prev_compass_mode: [["|",1,2],["-",1,2],["|",1,2]],
    key_next_compass_mode: [["\\",1,2],["=",1,2],["\\",1,2]],
    key_cloaking_device: [["0",1,5],["c",1,5],["0",1,5]],
    key_contract_info: [["?",10,4],["C",10,4],["?",10,4]],
    key_next_target: [["+",1,2],[";",1,2],["+",1,2]],
    key_previous_target: [["-",1,2],["'",1,2],["-",1,2]],
    key_custom_view: [["v",1,4],["V",1,4],["v",1,4]],
    key_dump_target_state: [["H",1,6],[250,1,6],["H",1,6]]
  };
  this.KC_StyleAtt = legendPadStyles; // Get styles for used keys
  this.KC_ClashTypes = []; // Holds clash types
  this.KC_AvailableKeys = []; // Holds key names
  this.KC_CurrentSet = {}; // Current configuration
  this.KC_Duplicates = []; // Holds only values from current config
  this.KC_RegKeys = /[ A-Za-z0-9@,\.\^!\$%&\/\(\)=\?\[\]\\`'#-_\|:]/g; // RegExp to check valid entries via direct keyboard input
  this.KC_setID = 0; // Set Nr - 0=Default, 1=Alternative, 2=BBC

  // Check JS, fill lists and change the values for Macs
  this.JSCheck = function(){
    if(VKI_isMoz) this.KC_AvailableKeys = Object.keys(this.KC_Default);
    else for(prop in this.KC_Default) this.KC_AvailableKeys.push(prop);
    if(document.getElementById){
      var e,f;
      e = document.getElementById("JSCheck");
      e.firstChild.nodeValue = "A quick and dirty tool to create your own keyconfig.plist. ";
      e.style.backgroundColor = "#E0F0E0";
      f = document.getElementById("ooliteHelpImage");
      f.src = "Data/keyconfig_help.png";
      f.alt = "Help";
      f.style.visibility = "visible";
      f = document.getElementById("ooliteHelp");
      f.style.visibility = "visible";
    }
    if(VKI_isMac){
      var t;
      for(mo=0;mo<this.KC_AvailableKeys.length;mo++){
        for(mi=0;mi<this.KC_Default[this.KC_AvailableKeys[mo]].length;mi++){
          t = this.KC_Default[this.KC_AvailableKeys[mo]][mi][0];
          switch(t){
            case 303: this.KC_Default[this.KC_AvailableKeys[mo]][mi][0] = "\u2196"; break;
            case 304: this.KC_Default[this.KC_AvailableKeys[mo]][mi][0] = "\u2198"; break;
            case 307: this.KC_Default[this.KC_AvailableKeys[mo]][mi][0] = "\u21de"; break;
            case 308: this.KC_Default[this.KC_AvailableKeys[mo]][mi][0] = "\u21df"; break;
          }
        }
      }
    }
    for(i=0;i<this.KC_AvailableKeys.length;i++){
      this.KC_CurrentSet[this.KC_AvailableKeys[i]] = this.KC_Default[this.KC_AvailableKeys[i]][this.KC_setID][0];
      this.KC_ClashTypes[i] = this.KC_Default[this.KC_AvailableKeys[i]][this.KC_setID][1];
    }
    document.getElementsByTagName("button")[0].className = "keyboardButtonActive";
    this.displaySet(1);
    return;
  };
  // Toggle help/usage
  this.showOoliteHelp = function(){
    var e;
    e = document.getElementById("keyboardUsage");
    if(e.style.visibility === "hidden") e.style.visibility = "visible";
    else e.style.visibility = "hidden";
    return;
  };
  // Change layout
  this.showLayout = function(n){
    this.KC_setID = n;
    for(i=0;i<this.KC_AvailableKeys.length;i++){
      this.KC_CurrentSet[this.KC_AvailableKeys[i]] = this.KC_Default[this.KC_AvailableKeys[i]][this.KC_setID][0];
      this.KC_ClashTypes[i] = this.KC_Default[this.KC_AvailableKeys[i]][this.KC_setID][1];
    }
    var e = document.getElementsByTagName("button");
    for(j=0;j<e.length;j++){
      if(n===j) e[j].className = "keyboardButtonActive";
      else e[j].className = "keyboardButton";
    }
    this.displaySet(1);
    return;
  };
  // Display settings
  this.displaySet = function(mod){
    VKI_deadkey = {};
    VKI_titles = {};
    this.KC_Duplicates = [];
    if(VKI_colorPreOn.checked){
      for(p=1;p<9;p++){
        var pf = p.toString();
        var pg = "F"+p;
        VKI_titles[pf] = ["Hardcoded Views/GUIs","deadkey",0];
        VKI_deadkey[pf] = {pf:pf};
        VKI_titles[pg] = ["Hardcoded Views/GUIs","deadkey",0];
        VKI_deadkey[pg] = {pg:pg};
      }
      VKI_titles["Q"] = ["Hardcoded QUIT","deadkey",0];
      VKI_deadkey["Q"] = {"Q":"Q"};
    }
    for(i=0;i<this.KC_AvailableKeys.length;i++){
      var t = this.KC_AvailableKeys[i];
      if(!this.KC_CurrentSet[t]) this.KC_CurrentSet[t] = "";
      var test = this.KC_getConv(escape(this.KC_CurrentSet[t]),1);
      document.getElementById(this.KC_AvailableKeys[i]).value = test;
      VKI_deadkey[test] = {test:test};
      var st = "deadkey",stn = 0;
      if(this.KC_Default[this.KC_AvailableKeys[i]][this.KC_setID].length>2){
        stn = this.KC_Default[this.KC_AvailableKeys[i]][this.KC_setID][2];
        st = this.KC_StyleAtt[stn];
      }
      if(!VKI_titles[test]) VKI_titles[test] = [this.KC_AvailableKeys[i],st,stn];
      else {
        VKI_titles[test][0] = VKI_titles[test][0]+", "+this.KC_AvailableKeys[i];
        if(!VKI_titles[test][2] || VKI_titles[test][2]>stn){
          VKI_titles[test][1] = st;
          VKI_titles[test][2] = stn;
        }
      }
      this.KC_Duplicates.push(test);
    }
    this.KC_MarkDuplicates();
    if(mod) VKI_modify();
    return;
  };
  // User input, onblur
  this.userKey = function(who,str){
    if(str===null || str.length===2) this.KC_CurrentSet[who][0] = this.KC_Default[who][this.KC_setID][0];
    else {
      if(str.search(this.KC_RegKeys)!==-1) this.KC_CurrentSet[who] = str;
      else this.KC_CurrentSet[who][0] = this.KC_Default[who][this.KC_setID][0];
    }
    this.displaySet();
    this.KC_MarkDuplicates();
    return;
  };
  // Quick User input, onkeyup. Fails when Alt+number is used.
  this.userCheckKey = function(who,str){
    if(str===null) document.getElementById(who).style.backgroundColor = "#FFFFFF";
    else {
      if(str.search(this.KC_RegKeys)===-1) document.getElementById(who).style.backgroundColor = "#FFC0C0";
      else document.getElementById(who).style.backgroundColor = "#FFFFFF";
    }
    return;
  };
  // Quick check for duplicated values
  this.arrHasDupes = function(Arr){
    var i,j,n,r=[];
    n=Arr.length;
    for(i=0;i<n;i++){
      for(j=i+1;j<n;j++){
        if(Arr[i]==="") r.push([i,i]);
        if (Arr[i]==Arr[j]) r.push([i,j]);
      }
    }
    return(r);
  };
  // Mark conflicts
  this.KC_MarkDuplicates = function(){
    for(b=0;b<this.KC_AvailableKeys.length;b++){
      document.getElementById(this.KC_AvailableKeys[b]).style.backgroundColor = "#FFFFFF";
    }
    var chk = this.arrHasDupes(this.KC_Duplicates);
    if(chk.length){
      var o = chk.length, e = [];
      for(i=0;i<o;i++){
        if((this.KC_ClashTypes[chk[i][0]]&this.KC_ClashTypes[chk[i][1]])){
          document.getElementById(this.KC_AvailableKeys[chk[i][0]]).style.backgroundColor = "#FFC0C0";
          document.getElementById(this.KC_AvailableKeys[chk[i][1]]).style.backgroundColor = "#FFC0C0";
          e.push(chk[i][0]);
          e.push(chk[i][1]);
        } else {
          if(this.VKI_isIE){ // indexOf workaround
            var a1=a2=0;
            if(e.length){
              for(te=0;te<e.length;te++){
                if(chk[i][0]===e[te]) a1 = 1;
                if(chk[i][1]===e[te]) a2 = 1;
              }
            }
            if(!a1) document.getElementById(this.KC_AvailableKeys[chk[i][0]]).style.backgroundColor = "#E0F0E0";
            if(!a2) document.getElementById(this.KC_AvailableKeys[chk[i][1]]).style.backgroundColor = "#E0F0E0";
          } else {
            if(!e.length || e.indexOf(chk[i][0])===-1) document.getElementById(this.KC_AvailableKeys[chk[i][0]]).style.backgroundColor = "#E0F0E0";
            if(!e.length || e.indexOf(chk[i][1])===-1) document.getElementById(this.KC_AvailableKeys[chk[i][1]]).style.backgroundColor = "#E0F0E0";
          }
        }
      }
    }
  };
  // Fill output textarea
  this.createOutput = function(){
    var txt = "", SDLflag = 0;
    for(i=0;i<this.KC_AvailableKeys.length;i++){
      var a = this.KC_AvailableKeys[i];
      if(!this.KC_CurrentSet[a] || this.KC_CurrentSet[a]===""){
        txt += "\t//";
        while(a.length<30) a += " ";
        txt += a + " = \"\";\n";
      } else {
        var b = escape(this.KC_CurrentSet[a]);
        var c = 0;
        if(typeof(this.KC_CurrentSet[a])==='number') c = 1;
        else {
          var d = this.KC_getConv(b,0);
          b = d[0];
          c = d[1];
          if(d[2]) SDLflag = d[2];
        }
        txt += "\t";
        while(a.length<30) a += " ";
        if(!c) txt += a + " = \"" + b + "\";\n";
        else txt += a + " = " + b + ";\n";
      }
    }
    txt += "}\n";
    if(SDLflag) txt = "{ // Keyconfig "+this.KC_Version+" for "+(VKI_isMac?"Mac":"SDL")+"\n"+txt;
    else txt = "{ // Keyconfig "+this.KC_Version+" - Generic \n"+txt;
    document.getElementsByName("output")[0].value = txt;
    this.KC_MarkDuplicates();
    return;
  };
  // Read-in textarea
  this.readIn = function(){
    var inp,vKey=false,vValue=false,num,bad=[],emflag=false,count=0;
    inp = document.getElementsByName("output")[0].value;
    var cont=inp,cKey,cSet={},f;
    if(inp){
      cont = cont.split("\n");
      for(o=0;o<cont.length;o++){
        if(RegExp(/^\s*$/).test(cont[o])) continue; // Strip empty lines
        if(RegExp("//").test(cont[o]) || RegExp("/*").test(cont[o])){ // Check comments
          f = cont[o].indexOf("//");
          if(f!==-1) cont[o] = cont[o].substr(0,f);
          f = cont[o].indexOf("/*");
          if(f!==-1) cont[o] = cont[o].substr(0,f);
        }
        cont[o] = cont[o].replace(/" ";/g,"32;"); // -> SPC
        cont[o] = cont[o].replace(/";";/g,"59;"); // -> Semicolon
        cont[o] = cont[o].replace(/"=";/g,"61;"); // -> Equals
        cont[o] = cont[o].replace(/[\s\{\};]/g,""); // Strip whitespaces, curly brackets and semicolon
        vKey=false;
        vValue=false;
        emflag=false;
        if(cont[o].search(/[;=]/g)===-1) continue;
        if(cont[o][0]==="{" || cont[o][0]==="/" || cont[o][0]==="}") continue;
        cKey = cont[o].split("=");
        if(cKey.length){
          cKey[0] = cKey[0].replace(/[\s"]/g,"");
          if(cKey.length>1) cKey[1] = cKey[1].replace(/[\s"]/g,"");
          for(av=0;av<this.KC_AvailableKeys.length;av++){
            if(this.KC_AvailableKeys[av]===cKey[0]){
              vKey = true;
              break;
            }
          }
          if(!vKey){
            bad.push(cKey);
            continue;
          }
          num = parseInt(cKey[1]);
          if(!isNaN(num) && typeof(num)==='number'){
            if((num>-1&&num<10) || num===27 || num===32 || (num>240&&num<256) || (num>300&&num<309)){
              cKey[1] = num;
              vValue = true;
            } else if(num===33 || (num>34&&num<123) || num===124){ // Check ASCII values
              cKey[1] = String.fromCharCode(num); // This way Keypad numbers are also converted
              vValue = true;
            } else {
              bad.push(cKey);
              emflag=true;
            }
          } else {
            if(cKey[1].length===1 && cKey[1].search(/[a-zA-Z@,\.\^!\$%&\/\(\)=\?\[\]\\`'#-_\|:;]/g)!==-1) vValue = true;
            if(cKey[1].length===2 && cKey[1].search(/[\t\\]/g)!==-1) vValue = true;
            if(!vValue){
              bad.push(""+cKey[0]+" : "+cKey[1]+" : "+cKey[1].length);
              continue;
            }
          }
          if(!emflag) cSet[cKey[0]] = this.KC_getConv(escape(cKey[1]),1);
        }
        count++;
      }
      if(bad.length) alert("Invalid input!\n\n"+bad+"\n\n");
      else if(count<51) alert("Pre v1.77 or incomplete keyconfig.\nMissing keys are marked as empty - output will comment out these keys.\n");
      this.KC_CurrentSet = cSet;
      var e = document.getElementsByTagName("button");
      for(j=0;j<e.length;j++) e[j].className = "keyboardButton";
      this.displaySet(1);
      document.getElementsByName("output")[0].value = null;
    }
    return;
  };
  this.KC_getConv = function(val, mode, log){
    var strA="",strB="",numflag=0,sdlflag=0;
    switch(val){
        case "%09": strA = "Tab"; strB = "\\t"; break; // Tab
        case "%20": strA = "SPC"; strB = 32; numflag = 1; break; // Space
        case "%21": strA = "!"; strB = "!"; break;
        case "%22": strA = "@"; strB = "@"; break; // " -> @
        case "%23": strA = "#"; strB = "#"; break;
        case "%24": strA = "$"; strB = "$"; break;
        case "%25": strA = "%"; strB = "%"; break;
        case "%26": strA = "&"; strB = "&"; break;
        case "%27": strA = "'"; strB = "'"; break;
        case "%28": strA = "("; strB = "("; break;
        case "%29": strA = ")"; strB = ")"; break;
        case "%2C": strA = ","; strB = ","; break;
        case "%3A": strA = ":"; strB = ":"; break;
        case "%3B": strA = ";"; strB = ";"; break;
        case "%3C": strA = "<"; strB = "<"; break;
        case "%3D": strA = "="; strB = "="; break;
        case "%3E": strA = ">"; strB = ">"; break;
        case "%3F": strA = "?"; strB = "?"; break;
        case "%5B": strA = "["; strB = "["; break;
        case "%5C": strA = "\\"; strB = "\\\\"; break;
        case "%5Ct": strA = "Tab"; strB = "\\t"; break;
        case "%5C%5C": strA = "\\"; strB = "\\\\"; break;
        case "%5D": strA = "]"; strB = "]"; break;
        case "%5E": strA = "^"; strB = "^"; break;
        case "%60": strA = "`"; strB = "`"; break;
        case "%7C": strA = "|"; strB = "|"; break;
        case "%u2196": strA = "\u2196"; strB = 303; numflag = 1; break; // Home, Mac
        case "%u2198": strA = "\u2198"; strB = 304; numflag = 1; break; // End, Mac
        case "%u21DE": strA = "\u21de"; strB = 307; numflag = 1; break; // Page up, Mac
        case "%u21DF": strA = "\u21df"; strB = 308; numflag = 1; break; // Page down, Mac
        case "241": strA = "F1"; break;
        case "242": strA = "F2"; break;
        case "243": strA = "F3"; break;
        case "244": strA = "F4"; break;
        case "245": strA = "F5"; break;
        case "246": strA = "F6"; break;
        case "247": strA = "F7"; break;
        case "248": strA = "F8"; break;
        case "249": strA = "F9"; break;
        case "250": strA = "F10"; break;
        case "251": strA = "F11"; break;
        case "252": strA = "CR\u02c3"; break; // Cursor left
        case "253": strA = "CR\u02c2"; break; // Cursor right
        case "254": strA = "CR\u02c5"; break; // Cursor up
        case "255": strA = "CR\u02c4"; break; // Cursor down
        case "27": strA = "ESC"; break;
        case "301": strA = "MBL"; break;
        case "302": strA = "MBD"; break;
        case "303": if(VKI_isMac) strA = "\u2196"; else strA = "HOM"; break;
        case "304": if(VKI_isMac) strA = "\u2198"; else strA = "END"; break;
        case "305": strA = "INS"; break;
        case "306": strA = "DEL"; break;
        case "307": if(VKI_isMac) strA = "\u21de"; else strA = "PU"; break;
        case "308": if(VKI_isMac) strA = "\u21df"; else strA = "PD"; break;
        case "32": strA = "SPC"; break;
        case "CR%u02C2": strA = "CR\u02c2"; strB = 253; numflag = 1; break; // Cursor left
        case "CR%u02C3": strA = "CR\u02c3"; strB = 252; numflag = 1; break; // Cursor right
        case "CR%u02C4": strA = "CR\u02c4"; strB = 255; numflag = 1; break; // Cursor up
        case "CR%u02C5": strA = "CR\u02c5"; strB = 254; numflag = 1; break; // Cursor down
        case "DEL": strA = "DEL"; strB = 306; numflag = 1; break;
        case "END": if(VKI_isMac) strA = "\u2198"; else strA = "END"; strB = 304; numflag = 1; break;
        case "ESC": strA = "ESC"; strB = 27; numflag = 1; break;
        case "F1": strA = "F1"; strB = 241; numflag = 1; break;
        case "F10": strA = "F10"; strB = 250; numflag = 1; break;
        case "F11": strA = "F11"; strB = 251; numflag = 1; break;
        case "F2": strA = "F2"; strB = 242; numflag = 1; break;
        case "F3": strA = "F3"; strB = 243; numflag = 1; break;
        case "F4": strA = "F4"; strB = 244; numflag = 1; break;
        case "F5": strA = "F5"; strB = 245; numflag = 1; break;
        case "F6": strA = "F6"; strB = 246; numflag = 1; break;
        case "F7": strA = "F7"; strB = 247; numflag = 1; break;
        case "F8": strA = "F8"; strB = 248; numflag = 1; break;
        case "F9": strA = "F9"; strB = 249; numflag = 1; break;
        case "HOM": if(VKI_isMac) strA = "\u2196"; else strA = "HOM"; strB = 303; numflag = 1; break;
        case "INS": strA = "INS"; strB = 305; numflag = 1; sdlflag = 1; break;
        case "MBL": strA = "MBL"; strB = 301; numflag = 1; break;
        case "MBD": strA = "MBD"; strB = 302; numflag = 1; break;
        case "PD": if(VKI_isMac) strA = "\u21df"; else strA = "PD"; strB = 308; numflag = 1; break;
        case "PU": if(VKI_isMac) strA = "\u21de"; else strA = "PU"; strB = 307; numflag = 1; break;
        case "SPC": strA = "SPC"; strB = 32; numflag = 1; break;
        case "Tab": strA = "Tab"; strB = "\\t"; break;
        default: strA = unescape(val); strB = unescape(val); if(log) alert(val);
    }
    if(mode) return(strA);
    return([strB,numflag,sdlflag]);
  };
  this.JSCheck();
  this.displaySet();
})();
